@@ -30,7 +30,11 @@ def _start_step(name, operation, step):
     logger.info("Executing " + str(flow_information['flowId']) + " step " + step)
     return_value = slims_instances[name]._execute_operation(operation, step, data)
     if return_value:
-        return jsonify(**return_value)
+        if isinstance(return_value, list):
+            # FIXME hack: jsonify(**varArgs) does not work for a list
+            return jsonify(return_value)
+        else:
+            return jsonify(**return_value)
     else:
         return jsonify(**{})
 
@@ -223,11 +227,13 @@ class Slims(object):
             Adds a content record with id "ID" in status pending with the content type
             with primary key 1
         """
-        response = self.slims_api.put(url=table, body=values).json()
-        new_values = response["entities"][0]
+        response = self.slims_api.put(url=table, body=values)
+        if response.status_code != 200:
+            raise _SlimsApiException("Add failed: " + response.text)
+        new_values = response.json()["entities"][0]
         return Record(new_values, self.slims_api)
 
-    def add_flow(self, flow_id, name, usage, steps, testing=False):
+    def add_flow(self, flow_id, name, usage, steps, testing=False, last_flow=True):
         """Add a new SLimsGate flow to the slims interface
 
         Note:
@@ -239,6 +245,7 @@ class Slims(object):
             name(string): Displayed name of the the flow
             usage(string): Usage of the slimsgate flow
             steps(list step): The steps of the slimsgate flow
+            last_flow(boolean): Defines if this is the last flow you will add (Default True)
 
         Examples:
             >>> def hello_world(flow_run):
@@ -263,12 +270,13 @@ class Slims(object):
 
         flow = {'id': flow_id, 'name': name, 'usage': usage, 'steps': step_dicts, 'pythonApiFlow': True}
         self.flow_definitions.append(flow)
-        self._register_flows([flow], False)
+        self._register_flows(self.flow_definitions, False)
 
         if not testing:
             if not self.refresh_flows_thread.is_alive():
                 self.refresh_flows_thread.start()
-            _flask_thread(self.local_port)
+            if last_flow:
+                _flask_thread(self.local_port)
 
     def _register_flows(self, flows, is_reregister):
         flow_ids = map(lambda flow: flow.get('id'), flows)
@@ -282,7 +290,7 @@ class Slims(object):
             if response.status_code == 200:
                 logger.info("Successfully " + verb + "ed " + str(flow_ids))
             else:
-                logger.info("Could not " + verb + " " + str(flow_ids) +
+                logger.info("Could not " + verb + " " + str(flow_ids) +  # noqa: W504
                             " (HTTP Response code: " + str(response.status_code) + ")")
                 try:
                     logger.info("Reason: " + response.json()["errorMessage"])
@@ -343,8 +351,10 @@ class Record(object):
             its id to "new id"
         """
         url = self.table_name() + "/" + str(self.pk())
-        response = self.slims_api.post(url=url, body=values).json()
-        new_values = response["entities"][0]
+        response = self.slims_api.post(url=url, body=values)
+        if response.status_code != 200:
+            raise _SlimsApiException("Update failed: " + response.text)
+        new_values = response.json()["entities"][0]
         return Record(new_values, self.slims_api)
 
     def remove(self):
@@ -354,7 +364,7 @@ class Record(object):
         url = self.table_name() + "/" + str(self.pk())
         response = self.slims_api.delete(url=url)
         if response.status_code != 200:
-            raise Exception("Delete failed: " + response.text)
+            raise _SlimsApiException("Delete failed: " + response.text)
 
     def table_name(self):
         """
